@@ -1,6 +1,6 @@
 
 /* 
- * Argyll Color Correction System
+ * Argyll Color Management System
  *
  * Xrite DTP41 related functions
  *
@@ -18,7 +18,7 @@
 
 /* 
    If you make use of the instrument driver code here, please note
-   that it is the author(s) of the code who take responsibility
+   that it is the author(s) of the code who are responsibility
    for its operation. Any problems or queries regarding driving
    instruments with the Argyll drivers, should be directed to
    the Argyll's author(s), and not to any other party.
@@ -612,6 +612,7 @@ ipatch *vals) {		/* Pointer to array of instrument patch values */
 			vals[i].mtype = inst_mrt_transmissive;
 		else
 			vals[i].mtype = inst_mrt_reflective;
+		vals[i].mcond = inst_mrc_none;
 		vals[i].XYZ_v = 1;
 		vals[i].sp.spec_n = 0;
 		vals[i].duration = 0.0;
@@ -620,7 +621,9 @@ ipatch *vals) {		/* Pointer to array of instrument patch values */
 
 	if (p->mode & inst_mode_spectral
 	 || (XCALSTD_NEEDED(p->target_calstd, p->native_calstd)
-		&& ((p->mode & inst_mode_illum_mask) != inst_mode_transmission))) {
+		&& ((p->mode & inst_mode_illum_mask) != inst_mode_transmission))
+	 || p->custfilt_en
+	) {
 
 		/* Gather the results in Spectral reflectance */
 		if ((ev = dtp41_command(p, "0403TS\r", buf, MAX_RD_SIZE, 0.5 + npatch * 0.1)) != inst_ok)
@@ -661,6 +664,10 @@ ipatch *vals) {		/* Pointer to array of instrument patch values */
 	ipatch_convert_xrga(vals, npatch, xcalstd_nonpol, p->target_calstd, p->native_calstd,
 	                    instClamp);
 
+	/* Apply custom filter compensation */
+	if (p->custfilt_en)
+		ipatch_convert_custom_filter(vals, npatch, &p->custfilt, instClamp);
+
 	if (user_trig)
 		return inst_user_trig;
 	return inst_ok;
@@ -808,6 +815,7 @@ instClamping clamp) {		/* NZ if clamp XYZ/Lab to be +ve */
 		val->mtype = inst_mrt_transmissive;
 	else
 		val->mtype = inst_mrt_reflective;
+	val->mcond = inst_mrc_none;
 	val->XYZ_v = 1;
 	val->sp.spec_n = 0;
 	val->duration = 0.0;
@@ -1295,6 +1303,42 @@ dtp41_get_set_opt(inst *pp, inst_opt_type m, ...)
 		return inst_ok;
 	}
 
+	if (m == inst_opt_set_custom_filter) {
+		va_list args;
+		xspect *sp = NULL;
+
+		va_start(args, m);
+
+		sp = va_arg(args, xspect *);
+
+		va_end(args);
+
+		if (sp == NULL || sp->spec_n == 0) {
+			p->custfilt_en = 0;
+			p->custfilt.spec_n = 0;
+		} else {
+			p->custfilt_en = 1;
+			p->custfilt = *sp;			/* Struct copy */
+		}
+		return inst_ok;
+	}
+
+	if (m == inst_stat_get_custom_filter) {
+		va_list args;
+		xspect *sp = NULL;
+
+		va_start(args, m);
+		sp = va_arg(args, xspect *);
+		va_end(args);
+
+		if (p->custfilt_en) {
+			*sp = p->custfilt;			/* Struct copy */
+		} else {
+			sp = NULL;
+		}
+		return inst_ok;
+	}
+
 	if (!p->gotcoms)
 		return inst_no_coms;
 	if (!p->inited)
