@@ -1,5 +1,5 @@
 /* 
- * Argyll Color Correction System
+ * Argyll Color Management System
  *
  * Xrite DTP92/94 related functions
  *
@@ -15,7 +15,7 @@
 
 /* 
    If you make use of the instrument driver code here, please note
-   that it is the author(s) of the code who take responsibility
+   that it is the author(s) of the code who are responsibility
    for its operation. Any problems or queries regarding driving
    instruments with the Argyll drivers, should be directed to
    the Argyll's author(s), and not to any other party.
@@ -641,6 +641,7 @@ instClamping clamp) {		/* NZ if clamp XYZ/Lab to be +ve */
 				icmClamp3(val->XYZ, val->XYZ);
 			val->loc[0] = '\000';
 			val->mtype = inst_mrt_emission;
+			val->mcond = inst_mrc_none;
 			val->XYZ_v = 1;		/* These are absolute XYZ readings */
 			val->sp.spec_n = 0;
 			val->duration = 0.0;
