@@ -1,6 +1,6 @@
 
 /* 
- * Argyll Color Correction System
+ * Argyll Color Management System
  *
  * Image Engineering EX1 related functions
  *
@@ -18,7 +18,7 @@
 
 /* 
    If you make use of the instrument driver code here, please note
-   that it is the author(s) of the code who take responsibility
+   that it is the author(s) of the code who are responsibility
    for its operation. Any problems or queries regarding driving
    instruments with the Argyll drivers, should be directed to
    the Argyll's author(s), and not to any other party.
@@ -382,7 +382,7 @@ ex1_init_inst(inst *pp) {
 	}
 #endif /* NEVER */
 
-	p->conv = new_xsp2cie(icxIT_none, NULL, icxOT_CIE_1931_2, NULL, icSigXYZData, icxNoClamp);
+	p->conv = new_xsp2cie(icxIT_none, 0.0, NULL, icxOT_CIE_1931_2, NULL, icSigXYZData, icxNoClamp);
 
 	if (p->conv == NULL)
 		return EX1_INT_CIECONVFAIL;
@@ -444,6 +444,7 @@ static int ex1_do_meas(ex1 *p, rspec **praw, double *inttime, double duration) {
 	}
 
 	sens->mtype = inst_mrt_emission;
+	sens->mcond = inst_mrc_none;
 	sens->inttime = p->inttime;
 
 	/* + Any other processing from sens to raw, */
@@ -645,6 +646,7 @@ instClamping clamp	/* NZ if clamp XYZ/Lab to be +ve */
 #endif
 
 	val->mtype = wav->mtype;
+	val->mcond = wav->mcond;
 
 	/* Copy spectral in */
 	val->sp.spec_n = sconf->nwav;
