@@ -2,7 +2,7 @@
 /* Abstract instrument class implemenation */
 
 /* 
- * Argyll Color Correction System
+ * Argyll Color Management System
  *
  * Author: Graeme W. Gill
  * Date:   10/3/2001
@@ -16,7 +16,7 @@
 
 /* 
    If you make use of the instrument driver code here, please note
-   that it is the author(s) of the code who take responsibility
+   that it is the author(s) of the code who are responsibility
    for its operation. Any problems or queries regarding driving
    instruments with the Argyll drivers, should be directed to
    the Argyll's author(s), and not to any other party.
@@ -41,11 +41,11 @@
 #ifndef SALONEINSTLIB
 #include "copyright.h"
 #include "aconfig.h"
+#include "rand.h"
 #else
 #include "sa_config.h"
 #endif /* !SALONEINSTLIB */
 #include "numsup.h"
-#include "rand.h"
 #include "cgats.h"
 #include "xspect.h"
 #include "conv.h"
@@ -463,16 +463,6 @@ double ref_rate) {
 	return inst_unsupported;
 }
 
-/* Insert a compensation filter in the instrument readings */
-/* This is typically needed if an adapter is being used, that alters */     
-/* the spectrum of the light reaching the instrument */                     
-/* To remove the filter, pass NULL for the filter filename */               
-static inst_code comp_filter(											        
-inst *p,
-char *filtername) {		/* File containing compensating filter */
-	return inst_unsupported;
-}
-
 /* Insert a colorimetric correction matrix in the instrument XYZ readings */
 /* This is only valid for colorimetric instruments. */
 /* To remove the matrix, pass NULL for the matrix */
@@ -689,6 +679,8 @@ void *cntx			/* Context for callback */
 	else if ((itype == instI1Pro) ||
 	         (itype == instI1Pro2))
 		p = (inst *)new_i1pro(icom, itype);
+	else if (itype == instI1Pro3)
+		p = (inst *)new_i1pro3(icom, itype);
 	else if (itype == instColorMunki)
 		p = (inst *)new_munki(icom, itype);
 	else if (itype == instSpyder1)
@@ -701,6 +693,8 @@ void *cntx			/* Context for callback */
 		p = (inst *)new_spyd2(icom, itype);
 	else if (itype == instSpyder5)
 		p = (inst *)new_spyd2(icom, itype);
+	else if (itype == instSpyderX)
+		p = (inst *)new_spydX(icom, itype);
 	else if (itype == instEX1)
 		p = (inst *)new_ex1(icom, itype);
 	if (itype == instHuey)
@@ -790,8 +784,6 @@ void *cntx			/* Context for callback */
 		p->get_refr_rate = get_refr_rate;
 	if (p->set_refr_rate == NULL)
 		p->set_refr_rate = set_refr_rate;
-	if (p->comp_filter == NULL)
-		p->comp_filter = comp_filter;
 	if (p->col_cor_mat == NULL)
 		p->col_cor_mat = col_cor_mat;
 	if (p->col_cal_spec_set == NULL)
@@ -881,6 +873,12 @@ static inst_disptypesel *expand_dlist(inst_disptypesel *list, int nlist, int *na
 	than any aliases that come after it, and the
 	aliases as more important than the fallback list,
 	so we need to do three passes through all the selections.
+
+	If we run out of single letter selectors (i.e. K10),
+	then we switch to the two letter prefixed selector "_X".
+
+	NOTE that we assume that we cannot exaust the single letter
+	selectors via hard coded, ccmx or ccss calibrations.
 */
 
 /* Create the display type list */
@@ -892,8 +890,8 @@ int doccss,						/* Add installed ccss files */
 int doccmx						/* Add matching installed ccmx files */
 ) {
 	inst_disptypesel *list = NULL;
-	int i, j, k, nlist = 0, nalist = 0;
-	char usels[256];			/* Used selectors */
+	int i, j, k, k2, nlist = 0, nalist = 0;
+	char usels[256];			/* Used selectors 1 */
 	static char *asels = "123456789abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ";
 	int fail = 0;
 
@@ -902,8 +900,9 @@ int doccmx						/* Add matching installed ccmx files */
 	*pdtlist = NULL;
 	*pndtlist = 0;
 
-	for (i = 0; i < 256; i++)
-		usels[i] = ((char)-1);
+	for (j = 0; j < 256; j++)
+		usels[j] = ((char)-1);
+	k2 = 0;		/* Single letter selector */
 	k = 0;		/* Next selector index */
 
 	/* First create a list of calibrations and their desired selectors: */
@@ -1031,26 +1030,42 @@ int doccmx						/* Add matching installed ccmx files */
 
 	/* Set remaining from fallback */
 	for (i = 0; i < nlist; i++) {
-		disptechs_set_sel(2, i, list[i].sel, list[i].isel, usels, &k, asels);
-		if (list[i].sel[0] == '\000')
-			fail = 1;
+		if (k2 == 0) {		/* Single letter selector */
+			disptechs_set_sel(2, i, list[i].sel, list[i].isel, usels, &k, asels);
+			if (list[i].sel[0] == '\000') {	/* Switch to two letter */
+				for (j = 0; j < 256; j++)
+					usels[j] = ((char)-1);
+				k2 = 1;		/* Two letter selector */
+				k = 0;		/* Next selector index */
+			}
+		} 
+		if (k2 == 1) {		/* Two letter selector */
+			list[i].sel[0] = '_';
+			disptechs_set_sel(2, i, &list[i].sel[1], list[i].isel, usels, &k, asels);
+			if (list[i].sel[0] == '\000') {	/* Ran out of two letter selectors! */
+				fail = 1;
+				break;
+			}
+		}
 	}
 
 	/* Any calibrations that failed to find a character will be left as a nul string */
 
 	/* Add alternate selectors if they are free. */
-	for (;;) {
-		int more = 0;
-		for (i = 0; i < nlist; i++) {
-			/* Add unused secondaries */
-			disptechs_set_sel(3, i, list[i].sel, list[i].isel, usels, &k, asels);
-
-			if (list[i].isel[0] != '\000') {		/* Still more secondaries available */
-				more = 1;
+	if (k2 == 0) {		/* If not run out of 1 letter selectors */
+		for (;;) {
+			int more = 0;
+			for (i = 0; i < nlist; i++) {
+				/* Add unused secondaries */
+				disptechs_set_sel(3, i, list[i].sel, list[i].isel, usels, &k, asels);
+	
+				if (list[i].isel[0] != '\000') {		/* Still more secondaries available */
+					more = 1;
+				}
 			}
+			if (!more)
+				break;
 		}
-		if (!more)
-			break;
 	}
 
 	if (pndtlist != NULL)
@@ -2116,6 +2131,52 @@ int sym_to_inst_mode(inst_mode *mode, const char *sym) {
 	return rv;
 }
 
+/* ============================================================= */
+/* Custom filter support */
+
+/* Apply a custom filer to an array of ipatch's */
+/* Spectral values are divided by filter spectrum and XYZ recomputed */
+void ipatch_convert_custom_filter(ipatch *vals, int nvals, xspect *filt, int clamp) {
+	xspect tmp;
+	xsp2cie *conv = NULL;	/* Spectral to XYZ conversion object */
+	int i, j;
+
+	/* If no conversion needed */
+	if (filt->spec_n == 0)
+		return;
+
+	for (i = 0; i < nvals; i++) {
+		if ((   vals[i].mtype != inst_mrt_emission
+		     && vals[i].mtype != inst_mrt_ambient
+		     && vals[i].mtype != inst_mrt_emission_flash
+		     && vals[i].mtype != inst_mrt_ambient_flash)
+		 || vals[i].sp.spec_n <= 0) {
+			continue;
+		}
+
+		/* Apply the filter compensation values */
+		for (j = 0; j < vals[i].sp.spec_n; j++) {
+			double nm = XSPECT_XWL(&vals[i].sp, j);
+			double fw;
+			getval_xspec(filt, &fw, nm);		/* Gets interp/extrap normalized value */
+			vals[i].sp.spec[j] /= fw;
+		}
+
+		/* Re-compute XYZ */
+		if (vals[i].XYZ_v) {
+			if (conv == NULL) {
+				conv = new_xsp2cie(icxIT_none, 0.0, NULL, icxOT_CIE_1931_2,
+				                   NULL, icSigXYZData, (icxClamping)clamp);
+			}
+			conv->convert(conv, vals[i].XYZ, &vals[i].sp);
+			vals[i].XYZ_v = 1;
+			vals[i].sp.norm = 1.0;
+		}
+	}
+	if (conv != NULL)
+		conv->del(conv);
+}
+
 
 
 
