@@ -12,7 +12,7 @@
  */
 
 /* 
- * Argyll Color Correction System
+ * Argyll Color Management System
  *
  * Author: Graeme W. Gill
  * Date:   15/3/2001
@@ -27,7 +27,7 @@
 
 /* 
    If you make use of the instrument driver code here, please note
-   that it is the author(s) of the code who take responsibility
+   that it is the author(s) of the code who are responsibility
    for its operation. Any problems or queries regarding driving
    instruments with the Argyll drivers, should be directed to
    the Argyll's author(s), and not to any other party.
@@ -43,8 +43,8 @@
    and agreed to support.
  */
 
-#include "dev.h"			/* Base device class */
 #include "insttypes.h"		/* libinst Includes this functionality */
+#include "dev.h"			/* Base device class */
 #include "disptechs.h"		/* libinst Includes this functionality */
 #include "icoms.h"			/* libinst Includes this functionality */
 #include "conv.h"
@@ -69,6 +69,14 @@
 
 #ifdef NEVER	/* Declared in xicc/xspect.h */
 
+// ~~~ should add absorbance mode -
+//     i.e. modified transmissive mode where units are
+//     log10(incident/transmitted) = Beer-Lambert Law
+
+/* Note that flash modes subtract background light */
+/* It's not clear if that is the best thing to do in terms */
+/* of either exposure or color temperature. */
+
 /* Type of measurement result */
 typedef enum {						/* XYZ units,      Spectral units */
 	inst_mrt_none           = 0,	/* Not set */
@@ -81,6 +89,20 @@ typedef enum {						/* XYZ units,      Spectral units */
 	inst_mrt_frequency      = 7		/* Hz */
 } inst_meas_type;
 
+/* Note that reflectance is measured under the given conditions, but that */
+/* by default XYZ is computed as if the spectral reflectance was illuminated */
+/* by a non-fluorescent stimulating D50 illuminant. */
+
+/* Reflective measurement conditions */
+typedef enum {
+	inst_mrc_none           = 0,	/* M0 - Default */
+	inst_mrc_D50            = 1,	/* M1 - D50 illuminant */
+	inst_mrc_D65            = 2,	/*      D65 Illuminant */
+	inst_mrc_uvcut          = 3,	/* M2 - U.V. Cut */ 
+	inst_mrc_pol            = 4,	/* M3 - Polarized */
+	inst_mrc_custom         = 5  	/*      Custom */
+} inst_meas_conditions;
+
 #endif // NEVER
 
 #define ICOM_MAX_LOC_LEN 10
@@ -88,7 +110,8 @@ typedef enum {						/* XYZ units,      Spectral units */
 struct _ipatch {
 	char loc[ICOM_MAX_LOC_LEN];	/* patch location */
 
-	inst_meas_type mtype;	/* Measurement type */
+	inst_meas_type mtype;		/* Measurement type */
+	inst_meas_cond mcond;		/* Reflective measurement conditions */
 	
 	int XYZ_v;				/* XYZ valid */
 	double XYZ[3];			/* XYZ values */
@@ -207,20 +230,20 @@ typedef enum {
 #   define inst_mode_emis_nonadaptive_sys       "EMNA"
 	inst_mode_ref_uv             = 0x00001000,	/* Ultra Violet measurement mode */
 #   define inst_mode_ref_uv_sym                 "REUV"
-	inst_mode_emis_refresh_ovd   = 0x00002000,	/* Emissom Refresh mode override */
+	inst_mode_emis_refresh_ovd   = 0x00002000,	/* Emission Refresh mode override */
 #   define inst_mode_emis_refresh_ovd_sym       "EMRO"
-	inst_mode_emis_norefresh_ovd = 0x00006000,	/* Emissom Non-refresh mode override */
+	inst_mode_emis_norefresh_ovd = 0x00006000,	/* Emission Non-refresh mode override */
 #   define inst_mode_emis_norefresh_ovd_sym     "ENRO"
 	inst_mode_dep_extra_mask     = 0x00007800,	/* Mask of measurement modifiers */
 
 	/* Extra independent modes */
-	inst_mode_colorimeter        = 0x00004000,	/* Colorimetric mode */
+	inst_mode_colorimeter        = 0x00010000,	/* Colorimetric mode */
 #   define inst_mode_colorimeter_sym            "COLI"
-	inst_mode_spectral           = 0x00008000,	/* Spectral mode */
+	inst_mode_spectral           = 0x00020000,	/* Spectral mode */
 #   define inst_mode_spectral_sym               "SPEC"
-	inst_mode_highres            = 0x00010000,	/* High Resolution Spectral mode */
+	inst_mode_highres            = 0x00040000,	/* High Resolution Spectral mode */
 #   define inst_mode_highres_sym                "HIRZ"
-	inst_mode_extra_mask         = 0x0001c000,	/* Mask of extra modes */
+	inst_mode_extra_mask         = 0x00070000,	/* Mask of extra modes */
 
 	/* Configured for calibration & capable of returning it from inst_mode_calibration */
 	inst_mode_calibration        = 0x80000000,	/* Configured for calibration */
@@ -350,9 +373,18 @@ typedef enum {
 typedef enum {
 	inst3_none              = 0x00000000, /* No capabilities */
 
-	inst3_average           = 0x00000001  /* Can set to average multiple measurements into 1 */
+	inst3_average           = 0x00000001, /* Can set to average multiple measurements into 1 */
 										  /* See inst_opt_set_averages */
 
+	/* inst_opt_set_filter() support: */
+	inst3_filter_none       = 0x00000010,	/* M0 - No filter */
+	inst3_filter_D50        = 0x00000020,	/* M1 - D50 illuminant filter */
+	inst3_filter_D65        = 0x00000040,	/*      D65 Illuminant filter */
+	inst3_filter_UVCut      = 0x00000080,	/* M2 - U.V. Cut filter */
+	inst3_filter_pol        = 0x00000100,	/* M3 - Polarising filter */ 
+	inst3_filter_Custom     = 0x00000200,	/* Manufacturers custom Filter */
+	inst3_filter_mask       = 0x000003F0 	/* Mask of all filter options */
+
 } inst3_capability;
 
 /* - - - - - - - - - - - - - - - - - - - */
@@ -417,7 +449,7 @@ typedef struct _inst_disptypesel {
 	// Stuff for ccss & ccmx
 	char *path;				/* Path to ccss or ccmx. NULL if not valid */
 	int cc_cbid;			/* cbid that matrix requires */
-	double mat[3][3];		/* ccmx matrix */
+	double mat[3][3];		/* ccmx or instrument matrix */
 	xspect *sets;   		/* ccss set of sample spectra. NULL if not valid */
 	int no_sets;    		/* ccs number of sets. 0 if not valid */
 
@@ -447,77 +479,89 @@ typedef enum {
 	inst_stat_battery           = 0x0006,	/* Return charged status of battery */
 											/* [1 argument type *double : range 0.0 - 1.0 ] */
 
-	inst_stat_get_filter        = 0x0007,	/* Get a filter configuration */
+	inst_stat_get_filter        = 0x0007,	/* Get a instrument filter configuration */
 											/* [1 argument type *inst_opt_filter ] */
 
-	inst_opt_initcalib          = 0x0008,	/* Enable initial calibration (default) [No args] */
-	inst_opt_noinitcalib        = 0x0009,	/* Disable initial calibration if < losecs since last */											/* opened, or losecs == 0 [int losecs] */
+	inst_stat_get_custom_filter = 0x0008,	/* Get a custom filter configuration. */
+											/* Will return NULL if none. */
+											/* [1 argument type xspect *] */
 
-	inst_opt_askcalib           = 0x000A,	/* Ask before proceeding with calibration (default) */
+	inst_opt_initcalib          = 0x0009,	/* Enable initial calibration (default) [No args] */
+	inst_opt_noinitcalib        = 0x000A,	/* Disable initial calibration if < losecs since last */											/* opened, or losecs == 0 [int losecs] */
+
+	inst_opt_askcalib           = 0x000B,	/* Ask before proceeding with calibration (default) */
 											/* [ No args] */
-	inst_opt_noaskcalib         = 0x000B,	/* Proceed with calibration immediately if possible */
+	inst_opt_noaskcalib         = 0x000C,	/* Proceed with calibration immediately if possible */
 											/* [ No args] */
 
-	inst_opt_set_ccss_obs       = 0x000C,	/* Set the observer used with ccss device types - */
+	inst_opt_set_ccss_obs       = 0x000D,	/* Set the observer used with ccss device types - */
 											/* Not applicable to any other type of instrument. */
 											/* [args: icxObserverType obType,*/
 	                                        /*        xspect custObserver[3] */
 
-	inst_opt_set_filter         = 0x000D,	/* Set a filter configuration */
+	inst_opt_set_filter         = 0x000E,	/* Set an instrument filter configuration */
 											/* [1 argument type inst_opt_filter] */
 
-	inst_opt_trig_prog          = 0x000E,	/* Trigger progromatically [No args] */
-	inst_opt_trig_user          = 0x000F,	/* Trigger from user via uicallback [No args] */
-	inst_opt_trig_switch        = 0x0010,	/* Trigger directly using instrument switch [No args] */
-	inst_opt_trig_user_switch   = 0x0011,	/* Trigger from user via uicallback or switch (def) [No args] */
+	inst_opt_set_custom_filter  = 0x000F,	/* Set a custom filter configuration. */
+											/* Emmissive mmnts. are divided by filter values. */
+											/* Can be set any time. */
+											/* Independent of inst_opt_set_filter. */
+											/* Set to NULL to dispable. */
+											/* [1 argument type xspect *] */
 
-	inst_opt_highres            = 0x0012,	/* Enable high res spectral mode indep. of set_mode() */
-	inst_opt_stdres             = 0x0013,	/* Revert to standard resolution spectral mode */
+	inst_opt_trig_prog          = 0x0010,	/* Trigger progromatically [No args] */
+	inst_opt_trig_user          = 0x0011,	/* Trigger from user via uicallback [No args] */
+	inst_opt_trig_switch        = 0x0012,	/* Trigger directly using instrument switch [No args] */
+	inst_opt_trig_user_switch   = 0x0013,	/* Trigger from user via uicallback or switch (def) [No args] */
 
-	inst_opt_scan_toll          = 0x0014,	/* Modify the patch scan recognition tollnce [double] */
+	inst_opt_highres            = 0x0014,	/* Enable high res spectral mode indep. of set_mode() */
+	inst_opt_stdres             = 0x0015,	/* Revert to standard resolution spectral mode */
 
-	inst_opt_get_gen_ledmask    = 0x0015,	/* Get the bitmask for general indication LEDs [*int] */
+	inst_opt_scan_toll          = 0x0016,	/* Modify the patch scan recognition tollnce [double] */
+
+	inst_opt_get_gen_ledmask    = 0x0017,	/* Get the bitmask for general indication LEDs [*int] */
 											/* (More specialized indicator masks go here) */
-	inst_opt_set_led_state      = 0x0016,	/* Set the current LED state. 0 = off, 1 == on [int] */
-	inst_opt_get_led_state      = 0x0017,	/* Get the current LED state. 0 = off, 1 == on [*int] */
-	inst_opt_get_pulse_ledmask  = 0x0018,	/* Get the bitmask for pulseable ind. LEDs [*int] */
-	inst_opt_set_led_pulse_state= 0x0019,	/* Set the current LED state. [double period_in_secs, */
+	inst_opt_set_led_state      = 0x0018,	/* Set the current LED state. 0 = off, 1 == on [int] */
+	inst_opt_get_led_state      = 0x0019,	/* Get the current LED state. 0 = off, 1 == on [*int] */
+	inst_opt_get_pulse_ledmask  = 0x001A,	/* Get the bitmask for pulseable ind. LEDs [*int] */
+	inst_opt_set_led_pulse_state= 0x001B,	/* Set the current LED state. [double period_in_secs, */
 	                                        /* double on_time_prop, double trans_time_prop] */
-	inst_opt_get_led_pulse_state= 0x001A,	/* Get the current pulse LED state. [*double period] */
-	inst_opt_get_target_state   = 0x001B,	/* Get the aiming target state 0 = off, 1 == on [*int] */
-	inst_opt_set_target_state   = 0x001C,	/* Set the aiming target state 0 = off, 1 == on, 2 = toggle [int] */
+	inst_opt_get_led_pulse_state= 0x001C,	/* Get the current pulse LED state. [*double period] */
+	inst_opt_get_target_state   = 0x001D,	/* Get the aiming target state 0 = off, 1 == on [*int] */
+	inst_opt_set_target_state   = 0x001E,	/* Set the aiming target state 0 = off, 1 == on, 2 = toggle [int] */
 
-	inst_opt_get_min_int_time   = 0x001D,	/* Get the minimum integration time [*double time] */
-	inst_opt_set_min_int_time   = 0x001E,	/* Set the minimum integration time [double time] */
+	inst_opt_get_min_int_time   = 0x001F,	/* Get the minimum integration time [*double time] */
+	inst_opt_set_min_int_time   = 0x0020,	/* Set the minimum integration time [double time] */
 
-	inst_opt_opt_calibs_valid   = 0x001F,	/* Are optional (white/black/gloss etc.) calibrations */
+	inst_opt_opt_calibs_valid   = 0x0021,	/* Are optional (white/black/gloss etc.) calibrations */
 	                                        /* valid?                     [*int valid] */
-	inst_opt_clear_opt_calibs   = 0x0020,	/* Clear all optional calibrations. */
+	inst_opt_clear_opt_calibs   = 0x0022,	/* Clear all optional calibrations. */
 
-	inst_opt_get_cal_tile_sp    = 0x0021,	/* Return refl. white tile reference spectrum. */
-	                                        /* for current filter.          [*xspect tile] */
+	inst_opt_get_cal_tile_sp    = 0x0023,	/* Return refl. white tile reference spectrum. */
+	                                        /* for current instrument filter. [*xspect tile] */
 
-	inst_opt_set_xcalstd        = 0x0022,	/* Set the X-Rite reflective calibration standard */
+	inst_opt_set_xcalstd        = 0x0024,	/* Set the X-Rite reflective calibration standard */
 											/*                             [xcalstd standard] */
-	inst_opt_get_xcalstd        = 0x0023,	/* Get the effective X-Rite ref. cal. standard */
+	inst_opt_get_xcalstd        = 0x0025,	/* Get the effective X-Rite ref. cal. standard */
 											/*                             [xcalstd *standard] */
-	inst_opt_lamp_remediate     = 0x0024,	/* Remediate i1Pro lamp           [double seconds] */
+	inst_opt_lamp_remediate     = 0x0026,	/* Remediate i1Pro lamp           [double seconds] */
 
-	inst_opt_set_averages       = 0x0025	/* Set the number of measurements to average [int] */
+	inst_opt_set_averages       = 0x0027	/* Set the number of measurements to average [int] */
 											/* 0 for default */
 
 
 } inst_opt_type;
 
-/* Optional filter fitted to instrument (for inst_opt_set_filter) */
-/* Set this before calling init_coms() */
+/* Optional manufacturers instrument filter fitted to instrument (for inst_opt_set_filter) */
+/* These could be physical (i.e. Spectrolino, i1Pro3), or they could be virtual (i.e. i1Pro3) */  
 typedef enum {
 	inst_opt_filter_unknown  = 0xffff,	/* Unspecified filter */
-	inst_opt_filter_none     = 0x0000,	/* No filters fitted */
-	inst_opt_filter_pol      = 0x0001,	/* Polarising filter */ 
-	inst_opt_filter_D65      = 0x0002,	/* D65 Illuminant filter */
-	inst_opt_filter_UVCut    = 0x0004,	/* U.V. Cut filter */
-	inst_opt_filter_Custom   = 0x0008	/* Custom Filter */
+	inst_opt_filter_none     = 0x0000,	/* M0 - No filters fitted */
+	inst_opt_filter_D50      = 0x0001,	/* M1 - D50 illuminant filtrr */
+	inst_opt_filter_D65      = 0x0002,	/*      D65 Illuminant filter */
+	inst_opt_filter_UVCut    = 0x0004,	/* M2 - U.V. Cut filter */
+	inst_opt_filter_pol      = 0x0008,	/* M3 - Polarising filter */ 
+	inst_opt_filter_Custom   = 0x0010	/* Manufacturers custom Filter */
 } inst_opt_filter;
 
 /* Off-line pending readings available (status) */
@@ -579,7 +623,7 @@ char *calt2str(inst_cal_type calt);
 /* This is how the instrument communicates to the calling program */
 /* about how to facilitate a calibration, or what it's current measurement */
 /* configuration provides. */
-/* [There is no provission for explictly indicating calibrations that can be */
+/* [There is no provision for explictly indicating calibrations that can be */
 /* performed automatically and transparently by the instrument - for instance */
 /* in the case of the spectroscan, since the required condition can be obtained */
 /* without the users interaction. ] */
@@ -634,11 +678,12 @@ typedef enum {
 	inst_calc_id_trans_low = 0x00010000,	/* Trans. Ref. light is too low for accuracy warning */
 	inst_calc_id_trans_wl  = 0x00020000,	/* Trans. Ref. light is low at some wavelengths warning */
 	inst_calc_id_filt_unkn = 0x00100000,	/* Request unknown filter */
-	inst_calc_id_filt_none = 0x00200000,	/* Request no filter */
-	inst_calc_id_filt_pol  = 0x00300000,	/* Request polarizing filter */
+	inst_calc_id_filt_none = 0x00200000,	/* Request no filter (M0) */
+	inst_calc_id_filt_D50  = 0x00300000,	/* Request D50 filter (M1) */
 	inst_calc_id_filt_D65  = 0x00400000,	/* Request D65 filter */
-	inst_calc_id_filt_UV   = 0x00500000,	/* Request UV cut filter */
-	inst_calc_id_filt_cust = 0x00600000		/* Request custom filter */
+	inst_calc_id_filt_UV   = 0x00500000,	/* Request UV cut filter (M2) */
+	inst_calc_id_filt_pol  = 0x00600000,	/* Request polarizing filter (M3) */
+	inst_calc_id_filt_cust = 0x00700000		/* Request instrument custom filter */
 } inst_calc_id_type;
 
 /* Clamping state */
@@ -659,7 +704,7 @@ typedef enum {
 /* Asynchronous event callback type */
 typedef enum {
     inst_event_switch,		/* Instrument measure/calibrate switch pressed. */
-    inst_event_mconf,		/* Change in measurement configuration (ie. sensor position) */
+    inst_event_mconf,		/* Change in measurement configuration (ie. sensor position, adapter) */
     inst_event_scan_ready	/* Ready for manual strip scan (i.e. issue audible prompt) */
 } inst_event_type;
 
@@ -668,8 +713,10 @@ typedef enum {
     inst_conf_unknown,
     inst_conf_projector,
     inst_conf_surface,	
+	inst_conf_surface_polarized,
     inst_conf_emission,	
 	inst_conf_calibration,
+	inst_conf_polarized_calibration,
 	inst_conf_ambient
 } inst_config;
 
@@ -1019,15 +1066,6 @@ typedef enum {
 		struct _inst *p,														\
 		double ref_rate);		/* Rate in Hz */								\
 																				\
-	/* Insert a compensation filter in the instrument readings */				\
-	/* This is typically needed if an adapter is being used, that alters */     \
-	/* the spectrum of the light reaching the instrument */                     \
-	/* Not fully implemented across all instrument types (spectrolino only ?) */ \
-	/* To remove the filter, pass NULL for the filter filename */               \
-	inst_code (*comp_filter)(											        \
-		struct _inst *p,														\
-		char *filtername);		/* File containing compensating filter */		\
-																				\
 	/* Insert a colorimetric correction matrix in the instrument XYZ readings */ \
 	/* This is only valid for colorimetric instruments, and can only be */		\
 	/* applied over a base calibration display type. Setting a display */		\
@@ -1110,7 +1148,7 @@ typedef enum {
 /* The base object type */
 struct _inst {
 	INST_OBJ_BASE
-	}; typedef struct _inst inst;
+}; typedef struct _inst inst;
 
 /* Virtual constructor. */
 /* Return NULL for unknown instrument, */
@@ -1201,6 +1239,13 @@ iccss *list_iccss(int *no);
 /* Free up a iccss list */
 void free_iccss(iccss *list);
 
+/* - - - - - - - - - - - - - - - - - - -- */
+/* Custom filter support */
+
+/* Apply a custom filer to an array of ipatch's */
+/* Spetral values are divided by filter spectrum and XYZ recomputed */
+void ipatch_convert_custom_filter(ipatch *vals, int nvals, xspect *filt, int clamp);
+
 /* - - - - - - - - - - - - - - - - - - -- */
 
 #ifdef __cplusplus
